@@ -1,5 +1,7 @@
 use std::borrow::Cow;
+use std::collections::HashMap;
 use std::sync::Arc;
+use std::sync::Mutex as StdMutex;
 use std::time::Duration;
 
 use anyhow::Result;
@@ -15,8 +17,12 @@ use axum::http::HeaderMap;
 use axum::http::StatusCode;
 use axum::http::header::AUTHORIZATION;
 use axum::routing::get;
+use codex_app_server_protocol::AppBranding;
 use codex_app_server_protocol::AppInfo;
 use codex_app_server_protocol::AppListUpdatedNotification;
+use codex_app_server_protocol::AppMetadata;
+use codex_app_server_protocol::AppReview;
+use codex_app_server_protocol::AppScreenshot;
 use codex_app_server_protocol::AppsListParams;
 use codex_app_server_protocol::AppsListResponse;
 use codex_app_server_protocol::JSONRPCError;
@@ -84,8 +90,12 @@ async fn list_apps_uses_thread_feature_flag_when_thread_id_is_provided() -> Resu
         logo_url: None,
         logo_url_dark: None,
         distribution_channel: None,
+        branding: None,
+        app_metadata: None,
+        labels: None,
         install_url: None,
         is_accessible: false,
+        is_enabled: true,
     }];
     let tools = vec![connector_tool("beta", "Beta App")?];
     let (server_url, server_handle) =
@@ -173,8 +183,116 @@ connectors = false
     Ok(())
 }
 
+#[tokio::test]
+async fn list_apps_reports_is_enabled_from_config() -> Result<()> {
+    let connectors = vec![AppInfo {
+        id: "beta".to_string(),
+        name: "Beta".to_string(),
+        description: Some("Beta connector".to_string()),
+        logo_url: None,
+        logo_url_dark: None,
+        distribution_channel: None,
+        branding: None,
+        app_metadata: None,
+        labels: None,
+        install_url: None,
+        is_accessible: false,
+        is_enabled: true,
+    }];
+    let tools = vec![connector_tool("beta", "Beta App")?];
+    let (server_url, server_handle) =
+        start_apps_server_with_delays(connectors, tools, Duration::ZERO, Duration::ZERO).await?;
+
+    let codex_home = TempDir::new()?;
+    std::fs::write(
+        codex_home.path().join("config.toml"),
+        format!(
+            r#"
+chatgpt_base_url = "{server_url}"
+
+[features]
+connectors = true
+
+[apps.beta]
+enabled = false
+"#
+        ),
+    )?;
+    write_chatgpt_auth(
+        codex_home.path(),
+        ChatGptAuthFixture::new("chatgpt-token")
+            .account_id("account-123")
+            .chatgpt_user_id("user-123")
+            .chatgpt_account_id("account-123"),
+        AuthCredentialsStoreMode::File,
+    )?;
+
+    let mut mcp = McpProcess::new(codex_home.path()).await?;
+    timeout(DEFAULT_TIMEOUT, mcp.initialize()).await??;
+
+    let request_id = mcp
+        .send_apps_list_request(AppsListParams {
+            limit: None,
+            cursor: None,
+            thread_id: None,
+            force_refetch: false,
+        })
+        .await?;
+
+    let response: JSONRPCResponse = timeout(
+        DEFAULT_TIMEOUT,
+        mcp.read_stream_until_response_message(RequestId::Integer(request_id)),
+    )
+    .await??;
+    let AppsListResponse {
+        data: response_data,
+        next_cursor,
+    } = to_response(response)?;
+    assert!(next_cursor.is_none());
+    assert_eq!(response_data.len(), 1);
+    assert_eq!(response_data[0].id, "beta");
+    assert!(!response_data[0].is_enabled);
+
+    server_handle.abort();
+    let _ = server_handle.await;
+    Ok(())
+}
+
 #[tokio::test]
 async fn list_apps_emits_updates_and_returns_after_both_lists_load() -> Result<()> {
+    let alpha_branding = Some(AppBranding {
+        category: Some("PRODUCTIVITY".to_string()),
+        developer: Some("Acme".to_string()),
+        website: Some("https://acme.example".to_string()),
+        privacy_policy: Some("https://acme.example/privacy".to_string()),
+        terms_of_service: Some("https://acme.example/terms".to_string()),
+        is_discoverable_app: true,
+    });
+    let alpha_app_metadata = Some(AppMetadata {
+        review: Some(AppReview {
+            status: "APPROVED".to_string(),
+        }),
+        categories: Some(vec!["PRODUCTIVITY".to_string()]),
+        sub_categories: Some(vec!["WRITING".to_string()]),
+        seo_description: Some("Alpha connector".to_string()),
+        screenshots: Some(vec![AppScreenshot {
+            url: Some("https://example.com/alpha-screenshot.png".to_string()),
+            file_id: Some("file_123".to_string()),
+            user_prompt: "Summarize this draft".to_string(),
+        }]),
+        developer: Some("Acme".to_string()),
+        version: Some("1.2.3".to_string()),
+        version_id: Some("version_123".to_string()),
+        version_notes: Some("Fixes and improvements".to_string()),
+        first_party_type: Some("internal".to_string()),
+        first_party_requires_install: Some(true),
+        show_in_composer_when_unlinked: Some(true),
+    });
+    let alpha_labels = Some(HashMap::from([
+        ("feature".to_string(), "beta".to_string()),
+        ("source".to_string(), "directory".to_string()),
+    ]));
+
     let connectors = vec![
         AppInfo {
             id: "alpha".to_string(),
@@ -183,8 +301,12 @@ async fn list_apps_emits_updates_and_returns_after_both_lists_load() -> Result<(
             logo_url: Some("https://example.com/alpha.png".to_string()),
             logo_url_dark: None,
             distribution_channel: None,
+            branding: alpha_branding.clone(),
+            app_metadata: alpha_app_metadata.clone(),
+            labels: alpha_labels.clone(),
             install_url: None,
             is_accessible: false,
+            is_enabled: true,
         },
         AppInfo {
             id: "beta".to_string(),
@@ -193,8 +315,12 @@ async fn list_apps_emits_updates_and_returns_after_both_lists_load() -> Result<(
             logo_url: None,
             logo_url_dark: None,
             distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
             install_url: None,
             is_accessible: false,
+            is_enabled: true,
         },
     ];
 
@@ -237,8 +363,12 @@ async fn list_apps_emits_updates_and_returns_after_both_lists_load() -> Result<(
         logo_url: None,
         logo_url_dark: None,
         distribution_channel: None,
+        branding: None,
+        app_metadata: None,
+        labels: None,
         install_url: Some("https://chatgpt.com/apps/beta-app/beta".to_string()),
         is_accessible: true,
+        is_enabled: true,
     }];
 
     let first_update = read_app_list_updated_notification(&mut mcp).await?;
@@ -252,8 +382,12 @@ async fn list_apps_emits_updates_and_returns_after_both_lists_load() -> Result<(
             logo_url: None,
             logo_url_dark: None,
             distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
             install_url: Some("https://chatgpt.com/apps/beta/beta".to_string()),
             is_accessible: true,
+            is_enabled: true,
         },
         AppInfo {
             id: "alpha".to_string(),
@@ -262,8 +396,12 @@ async fn list_apps_emits_updates_and_returns_after_both_lists_load() -> Result<(
             logo_url: Some("https://example.com/alpha.png".to_string()),
             logo_url_dark: None,
             distribution_channel: None,
+            branding: alpha_branding,
+            app_metadata: alpha_app_metadata,
+            labels: alpha_labels,
             install_url: Some("https://chatgpt.com/apps/alpha/alpha".to_string()),
             is_accessible: false,
+            is_enabled: true,
         },
     ];
 
@@ -298,8 +436,12 @@ async fn list_apps_returns_connectors_with_accessible_flags() -> Result<()> {
             logo_url: Some("https://example.com/alpha.png".to_string()),
             logo_url_dark: None,
             distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
             install_url: None,
             is_accessible: false,
+            is_enabled: true,
         },
         AppInfo {
             id: "beta".to_string(),
@@ -308,8 +450,12 @@ async fn list_apps_returns_connectors_with_accessible_flags() -> Result<()> {
             logo_url: None,
             logo_url_dark: None,
             distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
             install_url: None,
             is_accessible: false,
+            is_enabled: true,
         },
     ];
 
@@ -356,8 +502,12 @@ async fn list_apps_returns_connectors_with_accessible_flags() -> Result<()> {
                 logo_url: Some("https://example.com/alpha.png".to_string()),
                 logo_url_dark: None,
                 distribution_channel: None,
+                branding: None,
+                app_metadata: None,
+                labels: None,
                 install_url: Some("https://chatgpt.com/apps/alpha/alpha".to_string()),
                 is_accessible: false,
+                is_enabled: true,
             },
             AppInfo {
                 id: "beta".to_string(),
@@ -366,8 +516,12 @@ async fn list_apps_returns_connectors_with_accessible_flags() -> Result<()> {
                 logo_url: None,
                 logo_url_dark: None,
                 distribution_channel: None,
+                branding: None,
+                app_metadata: None,
+                labels: None,
                 install_url: Some("https://chatgpt.com/apps/beta/beta".to_string()),
                 is_accessible: false,
+                is_enabled: true,
             },
         ]
     );
@@ -380,8 +534,12 @@ async fn list_apps_returns_connectors_with_accessible_flags() -> Result<()> {
             logo_url: None,
             logo_url_dark: None,
             distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
             install_url: Some("https://chatgpt.com/apps/beta/beta".to_string()),
             is_accessible: true,
+            is_enabled: true,
         },
         AppInfo {
             id: "alpha".to_string(),
@@ -390,8 +548,12 @@ async fn list_apps_returns_connectors_with_accessible_flags() -> Result<()> {
             logo_url: Some("https://example.com/alpha.png".to_string()),
             logo_url_dark: None,
             distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
             install_url: Some("https://chatgpt.com/apps/alpha/alpha".to_string()),
             is_accessible: false,
+            is_enabled: true,
         },
     ];
 
@@ -421,8 +583,12 @@ async fn list_apps_paginates_results() -> Result<()> {
             logo_url: None,
             logo_url_dark: None,
             distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
             install_url: None,
             is_accessible: false,
+            is_enabled: true,
         },
         AppInfo {
             id: "beta".to_string(),
@@ -431,8 +597,12 @@ async fn list_apps_paginates_results() -> Result<()> {
             logo_url: None,
             logo_url_dark: None,
             distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
             install_url: None,
             is_accessible: false,
+            is_enabled: true,
         },
     ];
 
@@ -484,8 +654,12 @@ async fn list_apps_paginates_results() -> Result<()> {
         logo_url: None,
         logo_url_dark: None,
         distribution_channel: None,
+        branding: None,
+        app_metadata: None,
+        labels: None,
         install_url: Some("https://chatgpt.com/apps/beta/beta".to_string()),
         is_accessible: true,
+        is_enabled: true,
     }];
 
     assert_eq!(first_page, expected_first);
@@ -523,8 +697,12 @@ async fn list_apps_paginates_results() -> Result<()> {
         logo_url: None,
         logo_url_dark: None,
         distribution_channel: None,
+        branding: None,
+        app_metadata: None,
+        labels: None,
         install_url: Some("https://chatgpt.com/apps/alpha/alpha".to_string()),
         is_accessible: false,
+        is_enabled: true,
     }];
 
     assert_eq!(second_page, expected_second);
@@ -543,8 +721,12 @@ async fn list_apps_force_refetch_preserves_previous_cache_on_failure() -> Result
         logo_url: None,
         logo_url_dark: None,
         distribution_channel: None,
+        branding: None,
+        app_metadata: None,
+        labels: None,
         install_url: None,
         is_accessible: false,
+        is_enabled: true,
     }];
     let tools = vec![connector_tool("beta", "Beta App")?];
     let (server_url, server_handle) =
@@ -633,6 +815,228 @@ async fn list_apps_force_refetch_preserves_previous_cache_on_failure() -> Result
     Ok(())
 }
 
+#[tokio::test]
+async fn list_apps_force_refetch_patches_updates_from_cached_snapshots() -> Result<()> {
+    let initial_connectors = vec![
+        AppInfo {
+            id: "alpha".to_string(),
+            name: "Alpha".to_string(),
+            description: Some("Alpha v1".to_string()),
+            logo_url: None,
+            logo_url_dark: None,
+            distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
+            install_url: None,
+            is_accessible: false,
+            is_enabled: true,
+        },
+        AppInfo {
+            id: "beta".to_string(),
+            name: "Beta App".to_string(),
+            description: Some("Beta v1".to_string()),
+            logo_url: None,
+            logo_url_dark: None,
+            distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
+            install_url: None,
+            is_accessible: false,
+            is_enabled: true,
+        },
+    ];
+    let initial_tools = vec![connector_tool("beta", "Beta App")?];
+    let (server_url, server_handle, server_control) = start_apps_server_with_delays_and_control(
+        initial_connectors,
+        initial_tools,
+        Duration::from_millis(300),
+        Duration::ZERO,
+    )
+    .await?;
+
+    let codex_home = TempDir::new()?;
+    write_connectors_config(codex_home.path(), &server_url)?;
+    write_chatgpt_auth(
+        codex_home.path(),
+        ChatGptAuthFixture::new("chatgpt-token")
+            .account_id("account-123")
+            .chatgpt_user_id("user-123")
+            .chatgpt_account_id("account-123"),
+        AuthCredentialsStoreMode::File,
+    )?;
+
+    let mut mcp = McpProcess::new(codex_home.path()).await?;
+    timeout(DEFAULT_TIMEOUT, mcp.initialize()).await??;
+
+    let warm_request = mcp
+        .send_apps_list_request(AppsListParams {
+            limit: None,
+            cursor: None,
+            thread_id: None,
+            force_refetch: false,
+        })
+        .await?;
+    let warm_first_update = read_app_list_updated_notification(&mut mcp).await?;
+    assert_eq!(
+        warm_first_update.data,
+        vec![AppInfo {
+            id: "beta".to_string(),
+            name: "Beta App".to_string(),
+            description: None,
+            logo_url: None,
+            logo_url_dark: None,
+            distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
+            install_url: Some("https://chatgpt.com/apps/beta-app/beta".to_string()),
+            is_accessible: true,
+            is_enabled: true,
+        }]
+    );
+
+    let warm_second_update = read_app_list_updated_notification(&mut mcp).await?;
+    assert_eq!(
+        warm_second_update.data,
+        vec![
+            AppInfo {
+                id: "beta".to_string(),
+                name: "Beta App".to_string(),
+                description: Some("Beta v1".to_string()),
+                logo_url: None,
+                logo_url_dark: None,
+                distribution_channel: None,
+                branding: None,
+                app_metadata: None,
+                labels: None,
+                install_url: Some("https://chatgpt.com/apps/beta-app/beta".to_string()),
+                is_accessible: true,
+                is_enabled: true,
+            },
+            AppInfo {
+                id: "alpha".to_string(),
+                name: "Alpha".to_string(),
+                description: Some("Alpha v1".to_string()),
+                logo_url: None,
+                logo_url_dark: None,
+                distribution_channel: None,
+                branding: None,
+                app_metadata: None,
+                labels: None,
+                install_url: Some("https://chatgpt.com/apps/alpha/alpha".to_string()),
+                is_accessible: false,
+                is_enabled: true,
+            },
+        ]
+    );
+
+    let warm_response: JSONRPCResponse = timeout(
+        DEFAULT_TIMEOUT,
+        mcp.read_stream_until_response_message(RequestId::Integer(warm_request)),
+    )
+    .await??;
+    let AppsListResponse {
+        data: warm_data,
+        next_cursor: warm_next_cursor,
+    } = to_response(warm_response)?;
+    assert_eq!(warm_data, warm_second_update.data);
+    assert!(warm_next_cursor.is_none());
+
+    server_control.set_connectors(vec![AppInfo {
+        id: "alpha".to_string(),
+        name: "Alpha".to_string(),
+        description: Some("Alpha v2".to_string()),
+        logo_url: None,
+        logo_url_dark: None,
+        distribution_channel: None,
+        branding: None,
+        app_metadata: None,
+        labels: None,
+        install_url: None,
+        is_accessible: false,
+        is_enabled: true,
+    }]);
+    server_control.set_tools(Vec::new());
+
+    let refetch_request = mcp
+        .send_apps_list_request(AppsListParams {
+            limit: None,
+            cursor: None,
+            thread_id: None,
+            force_refetch: true,
+        })
+        .await?;
+
+    let first_update = read_app_list_updated_notification(&mut mcp).await?;
+    assert_eq!(
+        first_update.data,
+        vec![
+            AppInfo {
+                id: "alpha".to_string(),
+                name: "Alpha".to_string(),
+                description: Some("Alpha v1".to_string()),
+                logo_url: None,
+                logo_url_dark: None,
+                distribution_channel: None,
+                branding: None,
+                app_metadata: None,
+                labels: None,
+                install_url: Some("https://chatgpt.com/apps/alpha/alpha".to_string()),
+                is_accessible: false,
+                is_enabled: true,
+            },
+            AppInfo {
+                id: "beta".to_string(),
+                name: "Beta App".to_string(),
+                description: Some("Beta v1".to_string()),
+                logo_url: None,
+                logo_url_dark: None,
+                distribution_channel: None,
+                branding: None,
+                app_metadata: None,
+                labels: None,
+                install_url: Some("https://chatgpt.com/apps/beta-app/beta".to_string()),
+                is_accessible: false,
+                is_enabled: true,
+            },
+        ]
+    );
+
+    let expected_final = vec![AppInfo {
+        id: "alpha".to_string(),
+        name: "Alpha".to_string(),
+        description: Some("Alpha v2".to_string()),
+        logo_url: None,
+        logo_url_dark: None,
+        distribution_channel: None,
+        branding: None,
+        app_metadata: None,
+        labels: None,
+        install_url: Some("https://chatgpt.com/apps/alpha/alpha".to_string()),
+        is_accessible: false,
+        is_enabled: true,
+    }];
+    let second_update = read_app_list_updated_notification(&mut mcp).await?;
+    assert_eq!(second_update.data, expected_final);
+
+    let refetch_response: JSONRPCResponse = timeout(
+        DEFAULT_TIMEOUT,
+        mcp.read_stream_until_response_message(RequestId::Integer(refetch_request)),
+    )
+    .await??;
+    let AppsListResponse {
+        data: refetch_data,
+        next_cursor: refetch_next_cursor,
+    } = to_response(refetch_response)?;
+    assert_eq!(refetch_data, expected_final);
+    assert!(refetch_next_cursor.is_none());
+
+    server_handle.abort();
+    Ok(())
+}
+
 async fn read_app_list_updated_notification(
     mcp: &mut McpProcess,
 ) -> Result<AppListUpdatedNotification> {
@@ -652,22 +1056,46 @@ async fn read_app_list_updated_notification(
 struct AppsServerState {
     expected_bearer: String,
     expected_account_id: String,
-    response: serde_json::Value,
+    response: Arc<StdMutex<serde_json::Value>>,
     directory_delay: Duration,
 }
 
 #[derive(Clone)]
 struct AppListMcpServer {
-    tools: Arc<Vec<Tool>>,
+    tools: Arc<StdMutex<Vec<Tool>>>,
     tools_delay: Duration,
 }
 
 impl AppListMcpServer {
-    fn new(tools: Arc<Vec<Tool>>, tools_delay: Duration) -> Self {
+    fn new(tools: Arc<StdMutex<Vec<Tool>>>, tools_delay: Duration) -> Self {
         Self { tools, tools_delay }
     }
 }
 
+#[derive(Clone)]
+struct AppsServerControl {
+    response: Arc<StdMutex<serde_json::Value>>,
+    tools: Arc<StdMutex<Vec<Tool>>>,
+}
+
+impl AppsServerControl {
+    fn set_connectors(&self, connectors: Vec<AppInfo>) {
+        let mut response_guard = self
+            .response
+            .lock()
+            .unwrap_or_else(std::sync::PoisonError::into_inner);
+        *response_guard = json!({ "apps": connectors, "next_token": null });
+    }
+
+    fn set_tools(&self, tools: Vec<Tool>) {
+        let mut tools_guard = self
+            .tools
+            .lock()
+            .unwrap_or_else(std::sync::PoisonError::into_inner);
+        *tools_guard = tools;
+    }
+}
+
 impl ServerHandler for AppListMcpServer {
     fn get_info(&self) -> ServerInfo {
         ServerInfo {
@@ -688,8 +1116,12 @@ impl ServerHandler for AppListMcpServer {
             if tools_delay > Duration::ZERO {
                 tokio::time::sleep(tools_delay).await;
             }
+            let tools = tools
+                .lock()
+                .unwrap_or_else(std::sync::PoisonError::into_inner)
+                .clone();
             Ok(ListToolsResult {
-                tools: (*tools).clone(),
+                tools,
                 next_cursor: None,
                 meta: None,
             })
@@ -703,14 +1135,33 @@ async fn start_apps_server_with_delays(
     directory_delay: Duration,
     tools_delay: Duration,
 ) -> Result<(String, JoinHandle<()>)> {
+    let (server_url, server_handle, _server_control) =
+        start_apps_server_with_delays_and_control(connectors, tools, directory_delay, tools_delay)
+            .await?;
+    Ok((server_url, server_handle))
+}
+
+async fn start_apps_server_with_delays_and_control(
+    connectors: Vec<AppInfo>,
+    tools: Vec<Tool>,
+    directory_delay: Duration,
+    tools_delay: Duration,
+) -> Result<(String, JoinHandle<()>, AppsServerControl)> {
+    let response = Arc::new(StdMutex::new(
+        json!({ "apps": connectors, "next_token": null }),
+    ));
+    let tools = Arc::new(StdMutex::new(tools));
     let state = AppsServerState {
         expected_bearer: "Bearer chatgpt-token".to_string(),
         expected_account_id: "account-123".to_string(),
-        response: json!({ "apps": connectors, "next_token": null }),
+        response: response.clone(),
         directory_delay,
     };
     let state = Arc::new(state);
-    let tools = Arc::new(tools);
+    let server_control = AppsServerControl {
+        response,
+        tools: tools.clone(),
+    };
 
     let listener = TcpListener::bind("127.0.0.1:0").await?;
     let addr = listener.local_addr()?;
@@ -737,7 +1188,7 @@ async fn start_apps_server_with_delays(
         let _ = axum::serve(listener, router).await;
     });
 
-    Ok((format!("http://{addr}"), handle))
+    Ok((format!("http://{addr}"), handle, server_control))
 }
 
 async fn list_directory_connectors(
@@ -758,7 +1209,12 @@ async fn list_directory_connectors(
         .is_some_and(|value| value == state.expected_account_id);
 
     if bearer_ok && account_ok {
-        Ok(Json(state.response.clone()))
+        let response = state
+            .response
+            .lock()
+            .unwrap_or_else(std::sync::PoisonError::into_inner)
+            .clone();
+        Ok(Json(response))
     } else {
         Err(StatusCode::UNAUTHORIZED)
     }
