@@ -7,8 +7,11 @@ use std::time::Duration;
 use std::time::Instant;
 
 use async_channel::unbounded;
+pub use codex_app_server_protocol::AppBranding;
 pub use codex_app_server_protocol::AppInfo;
+pub use codex_app_server_protocol::AppMetadata;
 use codex_protocol::protocol::SandboxPolicy;
+use serde::Deserialize;
 use tokio_util::sync::CancellationToken;
 use tracing::warn;
 
@@ -16,6 +19,7 @@ use crate::AuthManager;
 use crate::CodexAuth;
 use crate::SandboxState;
 use crate::config::Config;
+use crate::config::types::AppsConfigToml;
 use crate::features::Feature;
 use crate::mcp::CODEX_APPS_MCP_SERVER_NAME;
 use crate::mcp::auth::compute_auth_statuses;
@@ -50,6 +54,19 @@ pub async fn list_accessible_connectors_from_mcp_tools(
     list_accessible_connectors_from_mcp_tools_with_options(config, false).await
 }
 
+pub async fn list_cached_accessible_connectors_from_mcp_tools(
+    config: &Config,
+) -> Option<Vec<AppInfo>> {
+    if !config.features.enabled(Feature::Apps) {
+        return Some(Vec::new());
+    }
+
+    let auth_manager = auth_manager_from_config(config);
+    let auth = auth_manager.auth().await;
+    let cache_key = accessible_connectors_cache_key(config, auth.as_ref());
+    read_cached_accessible_connectors(&cache_key)
+}
+
 pub async fn list_accessible_connectors_from_mcp_tools_with_options(
     config: &Config,
     force_refetch: bool,
@@ -265,6 +282,22 @@ pub fn merge_connectors(
     merged
 }
 
+pub fn with_app_enabled_state(mut connectors: Vec<AppInfo>, config: &Config) -> Vec<AppInfo> {
+    let apps = read_apps_config(config).map(|apps_config| apps_config.apps);
+    for connector in &mut connectors {
+        if let Some(app) = apps.as_ref().and_then(|apps| apps.get(&connector.id)) {
+            connector.is_enabled = app.enabled;
+        }
+    }
+    connectors
+}
+
+fn read_apps_config(config: &Config) -> Option<AppsConfigToml> {
+    let effective_config = config.config_layer_stack.effective_config();
+    let apps_config = effective_config.as_table()?.get("apps")?.clone();
+    AppsConfigToml::deserialize(apps_config).ok()
+}
+
 fn collect_accessible_connectors<I>(tools: I) -> Vec<AppInfo>
 where
     I: IntoIterator<Item = (String, Option<String>)>,
@@ -289,8 +322,12 @@ where
             logo_url: None,
             logo_url_dark: None,
             distribution_channel: None,
+            branding: None,
+            app_metadata: None,
+            labels: None,
             install_url: Some(connector_install_url(&connector_name, &connector_id)),
             is_accessible: true,
+            is_enabled: true,
         })
         .collect();
     accessible.sort_by(|left, right| {
