@@ -1,4 +1,5 @@
 use crate::exec::ExecToolCallOutput;
+use crate::network_policy_decision::NetworkPolicyDecisionPayload;
 use crate::token_data::KnownPlan;
 use crate::token_data::PlanType;
 use crate::truncate::TruncationPolicy;
@@ -31,7 +32,10 @@ pub enum SandboxErr {
         "sandbox denied exec error, exit code: {}, stdout: {}, stderr: {}",
         .output.exit_code, .output.stdout.text, .output.stderr.text
     )]
-    Denied { output: Box<ExecToolCallOutput> },
+    Denied {
+        output: Box<ExecToolCallOutput>,
+        network_policy_decision: Option<NetworkPolicyDecisionPayload>,
+    },
 
     /// Error from linux seccomp filter setup
     #[cfg(target_os = "linux")]
@@ -411,18 +415,22 @@ pub struct UsageLimitReachedError {
     pub(crate) resets_at: Option<DateTime<Utc>>,
     pub(crate) rate_limits: Option<Box<RateLimitSnapshot>>,
     pub(crate) promo_message: Option<String>,
-    pub(crate) limit_name: Option<String>,
 }
 
 impl std::fmt::Display for UsageLimitReachedError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        if let Some(limit_name) = self.limit_name.as_deref()
+        if let Some(limit_name) = self
+            .rate_limits
+            .as_ref()
+            .and_then(|snapshot| snapshot.limit_name.as_deref())
+            .map(str::trim)
+            .filter(|name| !name.is_empty())
             && !limit_name.eq_ignore_ascii_case("codex")
         {
             return write!(
                 f,
-                "You've hit your usage limit for {limit_name}.{}",
-                retry_suffix(self.resets_at.as_ref())
+                "You've hit your usage limit for {limit_name}. Switch to another model now,{}",
+                retry_suffix_after_or(self.resets_at.as_ref())
             );
         }
 
@@ -612,7 +620,7 @@ impl CodexErr {
 
 pub fn get_error_message_ui(e: &CodexErr) -> String {
     let message = match e {
-        CodexErr::Sandbox(SandboxErr::Denied { output }) => {
+        CodexErr::Sandbox(SandboxErr::Denied { output, .. }) => {
             let aggregated = output.aggregated_output.text.trim();
             if !aggregated.is_empty() {
                 output.aggregated_output.text.clone()
@@ -704,7 +712,6 @@ mod tests {
             resets_at: None,
             rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
-            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -733,6 +740,7 @@ mod tests {
         };
         let err = CodexErr::Sandbox(SandboxErr::Denied {
             output: Box::new(output),
+            network_policy_decision: None,
         });
         assert_eq!(get_error_message_ui(&err), "aggregate detail");
     }
@@ -749,6 +757,7 @@ mod tests {
         };
         let err = CodexErr::Sandbox(SandboxErr::Denied {
             output: Box::new(output),
+            network_policy_decision: None,
         });
         assert_eq!(get_error_message_ui(&err), "stderr detail\nstdout detail");
     }
@@ -765,6 +774,7 @@ mod tests {
         };
         let err = CodexErr::Sandbox(SandboxErr::Denied {
             output: Box::new(output),
+            network_policy_decision: None,
         });
         assert_eq!(get_error_message_ui(&err), "stdout only");
     }
@@ -808,6 +818,7 @@ mod tests {
         };
         let err = CodexErr::Sandbox(SandboxErr::Denied {
             output: Box::new(output),
+            network_policy_decision: None,
         });
         assert_eq!(
             get_error_message_ui(&err),
@@ -822,7 +833,6 @@ mod tests {
             resets_at: None,
             rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
-            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -837,7 +847,6 @@ mod tests {
             resets_at: None,
             rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
-            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -852,7 +861,6 @@ mod tests {
             resets_at: None,
             rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
-            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -871,7 +879,6 @@ mod tests {
                 resets_at: Some(resets_at),
                 rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
-                limit_name: None,
             };
             let expected = format!(
                 "You've hit your usage limit. To get more access now, send a request to your admin or try again at {expected_time}."
@@ -887,7 +894,6 @@ mod tests {
             resets_at: None,
             rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
-            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -902,7 +908,6 @@ mod tests {
             resets_at: None,
             rate_limits: Some(Box::new(rate_limit_snapshot())),
             promo_message: None,
-            limit_name: None,
         };
         assert_eq!(
             err.to_string(),
@@ -921,7 +926,6 @@ mod tests {
                 resets_at: Some(resets_at),
                 rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
-                limit_name: None,
             };
             let expected = format!(
                 "You've hit your usage limit. Visit https://chatgpt.com/codex/settings/usage to purchase more credits or try again at {expected_time}."
@@ -939,15 +943,18 @@ mod tests {
             let err = UsageLimitReachedError {
                 plan_type: Some(PlanType::Known(KnownPlan::Plus)),
                 resets_at: Some(resets_at),
-                rate_limits: Some(Box::new(rate_limit_snapshot())),
+                rate_limits: Some(Box::new(RateLimitSnapshot {
+                    limit_id: Some("codex_other".to_string()),
+                    limit_name: Some("codex_other".to_string()),
+                    ..rate_limit_snapshot()
+                })),
                 promo_message: Some(
                     "Visit https://chatgpt.com/codex/settings/usage to purchase more credits"
                         .to_string(),
                 ),
-                limit_name: Some("codex_other".to_string()),
             };
             let expected = format!(
-                "You've hit your usage limit for codex_other. Try again at {expected_time}."
+                "You've hit your usage limit for codex_other. Switch to another model now, or try again at {expected_time}."
             );
             assert_eq!(err.to_string(), expected);
         });
@@ -964,7 +971,6 @@ mod tests {
                 resets_at: Some(resets_at),
                 rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
-                limit_name: None,
             };
             let expected = format!("You've hit your usage limit. Try again at {expected_time}.");
             assert_eq!(err.to_string(), expected);
@@ -1074,7 +1080,6 @@ mod tests {
                 resets_at: Some(resets_at),
                 rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
-                limit_name: None,
             };
             let expected = format!(
                 "You've hit your usage limit. Upgrade to Pro (https://chatgpt.com/explore/pro), visit https://chatgpt.com/codex/settings/usage to purchase more credits or try again at {expected_time}."
@@ -1095,7 +1100,6 @@ mod tests {
                 resets_at: Some(resets_at),
                 rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
-                limit_name: None,
             };
             let expected = format!("You've hit your usage limit. Try again at {expected_time}.");
             assert_eq!(err.to_string(), expected);
@@ -1113,7 +1117,6 @@ mod tests {
                 resets_at: Some(resets_at),
                 rate_limits: Some(Box::new(rate_limit_snapshot())),
                 promo_message: None,
-                limit_name: None,
             };
             let expected = format!("You've hit your usage limit. Try again at {expected_time}.");
             assert_eq!(err.to_string(), expected);
@@ -1133,7 +1136,6 @@ mod tests {
                 promo_message: Some(
                     "To continue using Codex, start a free trial of <PLAN> today".to_string(),
                 ),
-                limit_name: None,
             };
             let expected = format!(
                 "You've hit your usage limit. To continue using Codex, start a free trial of <PLAN> today, or try again at {expected_time}."
