@@ -89,6 +89,9 @@ pub const DEFAULT_STARTUP_TIMEOUT: Duration = Duration::from_secs(10);
 const DEFAULT_TOOL_TIMEOUT: Duration = Duration::from_secs(60);
 
 const CODEX_APPS_TOOLS_CACHE_TTL: Duration = Duration::from_secs(3600);
+const MCP_TOOLS_LIST_DURATION_METRIC: &str = "codex.mcp.tools.list.duration_ms";
+const MCP_TOOLS_FETCH_UNCACHED_DURATION_METRIC: &str = "codex.mcp.tools.fetch_uncached.duration_ms";
+const MCP_TOOLS_CACHE_WRITE_DURATION_METRIC: &str = "codex.mcp.tools.cache_write.duration_ms";
 
 /// The Responses API requires tool names to match `^[a-zA-Z0-9_-]+$`.
 /// MCP server/tool names are user-controlled, so sanitize the fully-qualified
@@ -350,6 +353,10 @@ pub(crate) struct McpConnectionManager {
 }
 
 impl McpConnectionManager {
+    pub(crate) fn has_servers(&self) -> bool {
+        !self.clients.is_empty()
+    }
+
     pub async fn initialize(
         &mut self,
         mcp_servers: &HashMap<String, McpServerConfig>,
@@ -882,7 +889,7 @@ fn filter_tools(tools: Vec<ToolInfo>, filter: ToolFilter) -> Vec<ToolInfo> {
 }
 
 pub(crate) fn filter_codex_apps_mcp_tools_only(
-    mut mcp_tools: HashMap<String, ToolInfo>,
+    mcp_tools: &HashMap<String, ToolInfo>,
     connectors: &[crate::connectors::AppInfo],
 ) -> HashMap<String, ToolInfo> {
     let allowed: HashSet<&str> = connectors
@@ -890,26 +897,42 @@ pub(crate) fn filter_codex_apps_mcp_tools_only(
         .map(|connector| connector.id.as_str())
         .collect();
 
-    mcp_tools.retain(|_, tool| {
-        if tool.server_name != CODEX_APPS_MCP_SERVER_NAME {
-            return false;
-        }
-        let Some(connector_id) = tool.connector_id.as_deref() else {
-            return false;
-        };
-        allowed.contains(connector_id)
-    });
-
     mcp_tools
+        .iter()
+        .filter(|(_, tool)| {
+            if tool.server_name != CODEX_APPS_MCP_SERVER_NAME {
+                return false;
+            }
+            let Some(connector_id) = tool.connector_id.as_deref() else {
+                return false;
+            };
+            allowed.contains(connector_id)
+        })
+        .map(|(name, tool)| (name.clone(), tool.clone()))
+        .collect()
+}
+
+pub(crate) fn filter_non_codex_apps_mcp_tools_only(
+    mcp_tools: &HashMap<String, ToolInfo>,
+) -> HashMap<String, ToolInfo> {
+    mcp_tools
+        .iter()
+        .filter(|(_, tool)| tool.server_name != CODEX_APPS_MCP_SERVER_NAME)
+        .map(|(name, tool)| (name.clone(), tool.clone()))
+        .collect()
 }
 
 pub(crate) fn filter_mcp_tools_by_name(
-    mut mcp_tools: HashMap<String, ToolInfo>,
+    mcp_tools: &HashMap<String, ToolInfo>,
     selected_tools: &[String],
 ) -> HashMap<String, ToolInfo> {
     let allowed: HashSet<&str> = selected_tools.iter().map(String::as_str).collect();
-    mcp_tools.retain(|name, _| allowed.contains(name.as_str()));
+
     mcp_tools
+        .iter()
+        .filter(|(name, _)| allowed.contains(name.as_str()))
+        .map(|(name, tool)| (name.clone(), tool.clone()))
+        .collect()
 }
 
 fn normalize_codex_apps_tool_title(
@@ -1098,15 +1121,42 @@ async fn list_tools_for_client(
     client: &Arc<RmcpClient>,
     timeout: Option<Duration>,
 ) -> Result<Vec<ToolInfo>> {
+    let total_start = Instant::now();
     if server_name == CODEX_APPS_MCP_SERVER_NAME
         && let Some(cached_tools) = read_cached_codex_apps_tools()
     {
+        emit_duration(
+            MCP_TOOLS_LIST_DURATION_METRIC,
+            total_start.elapsed(),
+            &[("cache", "hit")],
+        );
         return Ok(cached_tools);
     }
 
+    let fetch_start = Instant::now();
     let tools = list_tools_for_client_uncached(server_name, client, timeout).await?;
+    emit_duration(
+        MCP_TOOLS_FETCH_UNCACHED_DURATION_METRIC,
+        fetch_start.elapsed(),
+        &[],
+    );
+
     if server_name == CODEX_APPS_MCP_SERVER_NAME {
+        let cache_write_start = Instant::now();
         write_cached_codex_apps_tools(&tools);
+        emit_duration(
+            MCP_TOOLS_CACHE_WRITE_DURATION_METRIC,
+            cache_write_start.elapsed(),
+            &[],
+        );
+    }
+
+    if server_name == CODEX_APPS_MCP_SERVER_NAME {
+        emit_duration(
+            MCP_TOOLS_LIST_DURATION_METRIC,
+            total_start.elapsed(),
+            &[("cache", "miss")],
+        );
     }
     Ok(tools)
 }
@@ -1123,7 +1173,12 @@ fn read_cached_codex_apps_tools() -> Option<Vec<ToolInfo>> {
         return Some(cached.tools.clone());
     }
 
-    *cache_guard = None;
+    if cache_guard
+        .as_ref()
+        .is_some_and(|cached| now >= cached.expires_at)
+    {
+        *cache_guard = None;
+    }
     None
 }
 
@@ -1137,6 +1192,12 @@ fn write_cached_codex_apps_tools(tools: &[ToolInfo]) {
     });
 }
 
+fn emit_duration(metric: &str, duration: Duration, tags: &[(&str, &str)]) {
+    if let Some(metrics) = codex_otel::metrics::global() {
+        let _ = metrics.record_duration(metric, duration, tags);
+    }
+}
+
 async fn list_tools_for_client_uncached(
     server_name: &str,
     client: &Arc<RmcpClient>,
@@ -1272,6 +1333,21 @@ mod tests {
         }
     }
 
+    fn with_clean_codex_apps_tools_cache<T>(f: impl FnOnce() -> T) -> T {
+        let previous_cache = {
+            let mut cache_guard = CODEX_APPS_TOOLS_CACHE
+                .lock()
+                .unwrap_or_else(std::sync::PoisonError::into_inner);
+            cache_guard.take()
+        };
+        let result = f();
+        let mut cache_guard = CODEX_APPS_TOOLS_CACHE
+            .lock()
+            .unwrap_or_else(std::sync::PoisonError::into_inner);
+        *cache_guard = previous_cache;
+        result
+    }
+
     #[test]
     fn test_qualify_tools_short_non_duplicated_names() {
         let tools = vec![
@@ -1424,6 +1500,47 @@ mod tests {
         assert_eq!(filtered[0].tool_name, "tool_a");
     }
 
+    #[test]
+    fn codex_apps_tools_cache_is_overwritten_by_last_write() {
+        with_clean_codex_apps_tools_cache(|| {
+            let tools_gateway_1 = vec![create_test_tool(CODEX_APPS_MCP_SERVER_NAME, "one")];
+            let tools_gateway_2 = vec![create_test_tool(CODEX_APPS_MCP_SERVER_NAME, "two")];
+
+            write_cached_codex_apps_tools(&tools_gateway_1);
+            let cached_gateway_1 =
+                read_cached_codex_apps_tools().expect("cache entry exists for first write");
+            assert_eq!(cached_gateway_1[0].tool_name, "one");
+
+            write_cached_codex_apps_tools(&tools_gateway_2);
+            let cached_gateway_2 =
+                read_cached_codex_apps_tools().expect("cache entry exists for second write");
+            assert_eq!(cached_gateway_2[0].tool_name, "two");
+        });
+    }
+
+    #[test]
+    fn codex_apps_tools_cache_is_cleared_when_expired() {
+        with_clean_codex_apps_tools_cache(|| {
+            let tools = vec![create_test_tool(CODEX_APPS_MCP_SERVER_NAME, "stale_tool")];
+            write_cached_codex_apps_tools(&tools);
+
+            {
+                let mut cache_guard = CODEX_APPS_TOOLS_CACHE
+                    .lock()
+                    .unwrap_or_else(std::sync::PoisonError::into_inner);
+                cache_guard.as_mut().expect("cache exists").expires_at =
+                    Instant::now() - Duration::from_secs(1);
+            }
+
+            assert!(read_cached_codex_apps_tools().is_none());
+
+            let cache_guard = CODEX_APPS_TOOLS_CACHE
+                .lock()
+                .unwrap_or_else(std::sync::PoisonError::into_inner);
+            assert!(cache_guard.is_none());
+        });
+    }
+
     #[test]
     fn mcp_init_error_display_prompts_for_github_pat() {
         let server_name = "github";
