@@ -1214,7 +1214,7 @@ async fn find_thread_path_by_id_str_in_subdir(
 
     let found = results.matches.into_iter().next().map(|m| m.full_path());
     if let Some(found_path) = found.as_ref() {
-        tracing::error!("state db missing rollout path for thread {id_str}");
+        tracing::debug!("state db missing rollout path for thread {id_str}");
         state_db::record_discrepancy("find_thread_path_by_id_str_in_subdir", "falling_back");
         state_db::read_repair_rollout_path(
             state_db_ctx.as_deref(),
