@@ -4,6 +4,7 @@ use std::path::Path;
 use std::path::PathBuf;
 
 use chrono::Utc;
+use codex_core::EventPersistenceMode;
 use codex_core::RolloutRecorder;
 use codex_core::RolloutRecorderParams;
 use codex_core::config::ConfigBuilder;
@@ -171,6 +172,7 @@ async fn find_locates_rollout_file_written_by_recorder() -> std::io::Result<()>
             SessionSource::Exec,
             BaseInstructions::default(),
             Vec::new(),
+            EventPersistenceMode::Limited,
         ),
         None,
         None,
