@@ -9,8 +9,9 @@ pub use types::Hook;
 pub use types::HookEvent;
 pub use types::HookEventAfterAgent;
 pub use types::HookEventAfterToolUse;
-pub use types::HookOutcome;
 pub use types::HookPayload;
+pub use types::HookResponse;
+pub use types::HookResult;
 pub use types::HookToolInput;
 pub use types::HookToolInputLocalShell;
 pub use types::HookToolKind;
