@@ -190,6 +190,7 @@ fn protocol_skill_to_core(skill: &ProtocolSkillMetadata) -> SkillMetadata {
                     .collect(),
             }),
         policy: None,
+        permissions: None,
         path: skill.path.clone(),
         scope: skill.scope,
     }
@@ -267,12 +268,12 @@ pub(crate) fn find_app_mentions(
     }
 
     let mut slug_counts: HashMap<String, usize> = HashMap::new();
-    for app in apps {
+    for app in apps.iter().filter(|app| app.is_enabled) {
         let slug = connector_mention_slug(app);
         *slug_counts.entry(slug).or_insert(0) += 1;
     }
 
-    for app in apps {
+    for app in apps.iter().filter(|app| app.is_enabled) {
         let slug = connector_mention_slug(app);
         let slug_count = slug_counts.get(&slug).copied().unwrap_or(0);
         if mentions.names.contains(&slug)
@@ -285,7 +286,7 @@ pub(crate) fn find_app_mentions(
     }
 
     apps.iter()
-        .filter(|app| selected_ids.contains(&app.id))
+        .filter(|app| app.is_enabled && selected_ids.contains(&app.id))
         .cloned()
         .collect()
 }
